grammar Drcql;

options {
	language = Java;
}

//@parser::header {  
//package guarnieri.marco.ethz.grammar;
//}
//
//@lexer::header {  
//package guarnieri.marco.ethz.grammar;
//}

/**
 * Grammar entry point. Represents all commands .
 */
command
:
	(
		'AS' u = ID ':' s = statement
	)+
;

/**
 * all statements
 */
statement
:
	booleanSelect
	| nonBooleanSelect
	| insert
	| delete
	| add_user
	| create_trigger
	| create_view
	| create_table
	| grant
	| grant_with_option
	| revoke
;
/**
 * boolean SELECT statements
 */
booleanSelect
:
	exp = expression
;

/**
 * non-boolean SELECT statements 
 */
nonBooleanSelect
:
	'{' var = varList '|' exp = expression '}'
;

/**
 * CREATE TRIGGER statement
 */
create_trigger
:
	'CREATE TRIGGER' id = ID 'AFTER' e = event 'AS' pmode = privilegeMode 'WHEN' w
	= expression 'DO'
	(
		actionI = insert
		| actionD = delete
		| actionG = grant
		| actionGO = grant_with_option
		| actionR = revoke
	)
;

/**
 * CREATE VIEW statement
 */
create_view
:
	'CREATE VIEW' id = ID '(' fields = varList ')' 'AS' pmode = privilegeMode 'OF'
	query = nonBooleanSelect
;

/**
 * CREATE TABLE statement
 */
create_table
:
	'CREATE TABLE' id = ID '(' fields = varList ')'
	(
		with = 'WITH' cstrs = constraintList
	)?
;

/**
 * List of integrity constraints.
 */
constraintList
:
	cstr = constraint
	| list = constraintList ',' cstr = constraint
;

/**
 * Integrity constraints
 */
constraint
:
	id = ID ':'
	(
		pk = 'PRIMARY KEY' '(' source = varList ')'
		| uq = 'UNIQUE' '(' source = varList ')'
		| fk = 'FOREIGN KEY' '(' source = varList ')' 'REFERENCES' table = ID '('
		target = varList ')'
	)
;

/**
 * ADD USER statement
 */
add_user
:
	'ADD USER' id = ID
;

/**
 * INSERT statement
 */
insert
:
	'INSERT IN' table = ID '[' tpl = tuple ']'
;

/**
 * DELETE statement
 */
delete
:
	'DELETE FROM' table = ID '[' tpl = tuple ']'
;

/**
 * Grant statements
 */
grant
:
	'GRANT' p = privilege 'TO' target = ID
;

/**
 * Grant* statements
 */
grant_with_option
:
	'GRANT*' p = privilege 'TO' target = ID
;

/**
 * Revoke statements
 */
revoke
:
	'REVOKE' p = privilege 'FROM' target = ID
;

/**
 * privileges
 */
privilege
:
	(
		s = 'SELECT ON'
		| i = 'INSERT ON'
		| d = 'DELETE ON'
		| ct = 'CREATE TRIGGER ON'
	) obj = ID
	| cv = 'CREATE VIEW'
;

privilegeMode
:
	ow = 'OWNER'
	| in = 'INVOKER'
;

/**
 * Tuple of values 
 */
tuple
:
	v = mixList
;

/**
 * List of variables and constants.
 */
mixList
:
	id = ID
	| list = mixList ',' id = ID
	| '\'' c = ID '\''
	| list = mixList ',' '\'' c = ID '\''
;

/**
 * List of variables. At the moment we consider just identifiers. It should be ok anyway.
 */
varList
:
	id = ID
	| list = varList ',' id = ID
;

/**
 * Trigger's event
 */
event
:
	(
		i = 'INSERT ON'
		| d = 'DELETE ON'
	) t = ID
;

/**
 * Relational calculus expression
 */
expression
:
	p = predicate
	| e = equality
	| '(' p1 = polyadicAndExpression ')'
	| '(' p2 = polyadicOrExpression ')'
	| u = unaryOpExpression
	| q = quantifiedExpression
	| f = 'FALSE'
	| t = 'TRUE'
;

quantifiedExpression
:
	(
		ex = 'EXISTS'
		| fa = 'FOR ALL'
	) var = varList '.' '(' expr = expression ')'
;

/**
 * Expressions with unary operators
 */
unaryOpExpression
:
	'NOT' '(' expr = expression ')'
;

/**
 * Expression with AND operator
 */
polyadicAndExpression
:
	exp1 = expression 'AND' exp2 = expression
	| other = polyadicAndExpression 'AND' exp2 = expression
;

/**
 * Expression with OR operator
 */
polyadicOrExpression
:
	exp1 = expression 'OR' exp2 = expression
	| other = polyadicOrExpression 'OR' exp2 = expression
;

/**
 * predicates. Note that we do not support constants in the predicates. Constants, however, can be used in equalities.
 */
predicate
:
	obj = ID '(' var = varList ')'
;

/**
 * equality
 */
equality
:
	var1 = ID '=' var2 = ID
	| '\'' const1 = ID '\'' '=' '\'' const2 = ID '\''
	| '\'' const1 = ID '\'' '=' var2 = ID
	| var1 = ID '=' '\'' const2 = ID '\''
;

/**
 * Identifiers
 */
ID
:
	(
		'a' .. 'z'
		| 'A' .. 'Z'
		| '_'
	)
	(
		'a' .. 'z'
		| 'A' .. 'Z'
		| '0' .. '9'
		| '_'
	)*
;

/**
 * We ignore all the white space characters
 */
WS
:
	(
		'\t'
		| '\r'
		| '\n'
		| ' '
	)+ -> skip
;


