package prototype.interfaces;

import prototype.results.actionResults.IActionResult;
import prototype.system.implementation.ExecutionException;
import prototype.system.interfaces.IDatabaseEngine;
import prototype.system.interfaces.IState;
import prototype.util.Pair;

/**
 * Interface for all executable elements, which can be either statements or objects.
 * 
 * 
 */
public interface IExecutable {

	/**
	 * Execute the statement and return its result.
	 * 
	 * @param engine
	 *            - The execution engine.
	 * @param state
	 *            - the current system's state.
	 * @return The statement's result and the resulting state.
	 * @throws ExecutionException 
	 */
	public Pair<IActionResult,IState> execute(IDatabaseEngine engine, IState state) throws ExecutionException;

}
