package prototype.objects;

import prototype.interfaces.CopyException;
import prototype.interfaces.ICopiable;
import prototype.users.IUser;

/**
 * Common interface for tables, triggers, and views.
 * 
 * 
 * 
 *
 */
public interface IDbObject extends Comparable<IDbObject>, ICopiable {

	/**
	 * Get the privilege mode associated with the object.
	 * 
	 * @return The object's privilege mode.
	 */
	public PRIVILEGE_MODE getMode();

	/**
	 * Get the object's owner.
	 * 
	 * @return The object's owner.
	 */
	public IUser getOwner();

	/**
	 * Get the object's unique identifier.
	 * 
	 * @return The object's Id.
	 */
	public String getId();

	/**
	 * Get the string representation of the object's definition.
	 * 
	 * @return The object's definition.
	 */
	public String getDefinition();
	
	public IDbObject copy() throws  CopyException;

}
