package prototype.objects;

import prototype.interfaces.CopyException;
import prototype.objects.implementation.triggers.actions.ITriggerAction;
import prototype.objects.implementation.triggers.events.ITriggerEvent;
import prototype.queries.domain.relational.calculus.syntax.IExpression;
import prototype.statements.MalformedStatementException;
import prototype.statements.abstractImplementation.IStatementFactory;
import prototype.statements.interfaces.IStatement;
import prototype.statements.interfaces.select.ISelectStatement;
import prototype.tuple.ITuple;
import prototype.users.IUser;

/**
 * Interface for triggers.
 * 
 * 
 *
 */
public interface ITrigger extends IDbObject {

	/**
	 * Get the trigger's event.
	 * 
	 * @return Trigger's event.
	 */
	public ITriggerEvent getEvent();

	/**
	 * Get the expression representing the trigger's condition. Note that it might contain free variables.
	 * 
	 * @return The triger's condition.
	 */
	public IExpression getCondition();

	/**
	 * Get the trigger's action.
	 * 
	 * @return The trigger's action.
	 */
	public ITriggerAction getAction();

	/**
	 * Get the concrete condition.
	 * 
	 * @param triggerInvoker
	 *            - The user invoking the trigger.
	 * @param concreteTuple
	 *            - The concrete tuple.
	 * @param factory
	 *            - The statement factory.
	 * @return The concrete SELECT statement.
	 * @throws MalformedStatementException
	 */
	public ISelectStatement getConcreteCondition(IUser triggerInvoker, ITuple concreteTuple, IStatementFactory factory)
			throws MalformedStatementException;

	/**
	 * Get the concrete action.
	 * 
	 * @param triggerInvoker
	 *            - The user invoking the trigger.
	 * @param concreteTuple
	 *            The concrete tuple.
	 * @param factory
	 *            - The statement factory.
	 * @return The concrete statement representing the action.
	 * @throws MalformedStatementException
	 */
	public IStatement getConcreteAction(IUser triggerInvoker, ITuple concreteTuple, IStatementFactory factory) throws MalformedStatementException;
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.objects.IDbObject#copy()
	 */
	public ITrigger copy() throws CopyException;

}
