package prototype.objects.implementation;

import prototype.interfaces.CopyException;
import prototype.objects.IDbObject;
import prototype.objects.MalformedDbObjectException;
import prototype.objects.PRIVILEGE_MODE;
import prototype.users.IUser;

/**
 * Abstract class for database objects.
 * 
 * 
 *
 */
public abstract class AbstractObject implements IDbObject {

	private PRIVILEGE_MODE mode;

	/**
	 * The object's id.
	 */
	private String id;

	/**
	 * The object's owner.
	 */
	private IUser owner;

	/**
	 * Constructor.
	 * 
	 * @param id
	 *            - The object's id.
	 * @param owner
	 *            - The object's owner.
	 * @param mode
	 *            - The privilege's mode.
	 * @throws MalformedDbObjectException
	 */
	public AbstractObject(String id, IUser owner, PRIVILEGE_MODE mode) throws MalformedDbObjectException {
		if (id != null && !id.isEmpty())
			if (owner != null)
				if (mode != null) {
					this.id = id;
					try {
						this.owner = owner.copy();
					} catch (CopyException e) {
						throw new MalformedDbObjectException(e);
					}
					this.mode = mode;
				} else
					throw new MalformedDbObjectException("Mode cannot be null.");
			else
				throw new MalformedDbObjectException("User cannot be null.");
		else
			throw new MalformedDbObjectException("Id cannot be null or empty.");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.objects.IDbObject#getMode()
	 */
	@Override
	public PRIVILEGE_MODE getMode() {
		return this.mode;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.objects.IDbObject#getOwner()
	 */
	@Override
	public IUser getOwner() {
		return owner;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.objects.IDbObject#getId()
	 */
	@Override
	public String getId() {
		return this.id;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.objects.IDbObject#getDefinition()
	 */
	@Override
	public abstract String getDefinition();

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(IDbObject o) {
		return this.getId().compareTo(o.getId());
	}
	
	@Override
	public String toString() {
		return getDefinition();
	}

}
