package prototype.objects.implementation;

import java.util.List;

import prototype.objects.IType;
import prototype.objects.MalformedDbObjectException;
import prototype.objects.PRIVILEGE_MODE;
import prototype.users.IUser;

/**
 * Abstract class for typed objects (such as tables or views).
 * 
 * 
 *
 */
public abstract class AbstractTypedObject extends AbstractObject implements IType {

	/**
	 * Type of the object.
	 */
	private IType type;

	/**
	 * Constructor.
	 * 
	 * @param id
	 *            - object's id.
	 * @param owner
	 *            - object's owner.
	 * @param mode
	 *            - object's mode.
	 * @param fields
	 *            - object's fields.
	 * @throws MalformedDbObjectException
	 */
	public AbstractTypedObject(String id, IUser owner, PRIVILEGE_MODE mode, List<String> fields) throws MalformedDbObjectException {
		super(id, owner, mode);
		this.type = new BasicType(id, fields);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.objects.IType#getFields()
	 */
	@Override
	public List<String> getFields() {
		return this.type.getFields();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.objects.IType#getCardinality()
	 */
	@Override
	public int getCardinality() {
		return this.type.getCardinality();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.objects.implementation.AbstractObject#getDefinition()
	 */
	@Override
	public abstract String getDefinition();

}
