package prototype.objects.implementation;

import java.util.List;

import prototype.objects.MalformedDbObjectException;
import prototype.queries.domain.relational.calculus.NonBooleanQuery;

/**
 * Class representing anonymous views.
 * 
 * 
 *
 */
public class AnonymousView extends BasicType {

	/**
	 * Query defining the view.
	 */
	private NonBooleanQuery definition;

	/**
	 * Constructor.
	 * 
	 * @param id
	 *            - View's identifier.
	 * 
	 * @param fields
	 *            - View's fields.
	 * @param definition
	 *            - View's definition.
	 * @throws MalformedDbObjectException
	 */
	public AnonymousView(String id, List<String> fields, NonBooleanQuery definition) throws MalformedDbObjectException {
		super(id, fields);
		if (definition != null) {

			if (definition instanceof NonBooleanQuery && ((NonBooleanQuery) definition).getOutput().size() == fields.size())
				this.definition = definition;

			else
				throw new MalformedDbObjectException("The fields in definition and in fields do not match (different size).");

		} else
			throw new MalformedDbObjectException("Definition cannot be null.");
	}

	public NonBooleanQuery getSelectQuery() {
		return this.definition;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return getId() + ":=" + this.definition;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		else if (obj instanceof AnonymousView) {
			if (obj == this)
				return true;
			else
				return ((AnonymousView) obj).getId().equals(this.getId()) && ((AnonymousView) obj).getFields().equals(this.getFields())
						&& ((AnonymousView) obj).getSelectQuery().equals(this.getSelectQuery());
		} else
			return false;
	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 31 * result + getId().hashCode();
		result = 31 * result + getFields().hashCode();
		result = 31 * result + getSelectQuery().hashCode();
		return result;
	}

}
