package prototype.objects.implementation.triggers.actions;

import prototype.interfaces.CopyException;
import prototype.interfaces.ICopiable;
import prototype.statements.MalformedStatementException;
import prototype.statements.abstractImplementation.IStatementFactory;
import prototype.statements.interfaces.IStatement;
import prototype.tuple.ITuple;
import prototype.users.IUser;

/**
 * Interface representing trigger's actions.
 * 
 * 
 *
 */
public interface ITriggerAction extends ICopiable {

	/**
	 * Get the concrete action.
	 * 
	 * @param user
	 *            - The user executing the action.
	 * @param concreteTuple
	 *            - The concrete tuple.
	 * @param factory
	 *            - The statement factory.
	 * @return The concrete statement.
	 */
	public IStatement concretize(IUser user, ITuple concreteTuple, IStatementFactory factory) throws MalformedStatementException;

	/**
	 * Get the action's definition.
	 * 
	 * @return Definition.
	 */
	public String getDefinition();
	
	
	/* (non-Javadoc)
	 * @see guarnieri.marco.ethz.interfaces.ICopiable#copy()
	 */
	public ITriggerAction copy() throws CopyException;
	
	

}
