package prototype.objects.implementation.triggers.events;

import prototype.interfaces.CopyException;
import prototype.objects.ITable;
import prototype.objects.MalformedDbObjectException;

/**
 * Class representing trigger events.
 * 
 * 
 *
 */
public class TriggerEvent implements ITriggerEvent {

	/**
	 * The time indicator, either AFTER or BEFORE.
	 */
	private TIME time;

	/**
	 * The type of the event.
	 */
	private EVENT eventType;

	/**
	 * The table on which the event is defined.
	 */
	private ITable on;

	/**
	 * Constructor.
	 * 
	 * @param time
	 *            - event's time.
	 * @param eventType
	 *            - event's type.
	 * @param on
	 *            - event's table.
	 * @throws MalformedDbObjectException
	 */
	public TriggerEvent(TIME time, EVENT eventType, ITable on) throws MalformedDbObjectException {
		if (time != null)
			if (eventType != null)
				if (on != null) {
					this.time = time;
					this.eventType = eventType;
					try {
						this.on = on.copy();
					} catch (CopyException e) {
						throw new MalformedDbObjectException(e);
					}
				} else
					throw new MalformedDbObjectException("On cannot be null.");
			else
				throw new MalformedDbObjectException("EventType cannot be null.");
		else
			throw new MalformedDbObjectException("Time cannot be null.");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.objects.implementation.triggers.events.ITriggerEvent#getTime()
	 */
	@Override
	public TIME getTime() {
		return this.time;
	}

	@Override
	public EVENT getEvent() {
		return eventType;
	}

	@Override
	public ITable getRelationSchema() {
		return on;
	}

	@Override
	public ITriggerEvent copy() throws CopyException {
		try {
			return new TriggerEvent(getTime(), getEvent(), getRelationSchema().copy());
		} catch (MalformedDbObjectException e) {
			throw new CopyException(e);
		}
	}

	@Override
	public String getDefinition() {
		return this.time + " " + this.eventType + " " + this.on.getId();
	}

	@Override
	public String toString() {
		return this.getDefinition();
	}
}
