package prototype.privileges;

import prototype.objects.IType;

/**
 * Abstract class representing privileges of the form PRIVILEGE ON TABLE.
 * 
 * 
 *
 */
public abstract class AbstractPrivilege implements IPrivilege {

	/**
	 * The privilege type.
	 */
	private PRIVILEGE privilege;
	/**
	 * The object on which the privilege is expressed.
	 */
	private IType obj;

	/**
	 * Constructor.
	 * 
	 * @param privilege
	 *            - privilege keyword.
	 * @param obj
	 *            - The object
	 * @throws MalformedPrivilege
	 */
	public AbstractPrivilege(PRIVILEGE privilege, IType obj) throws MalformedPrivilege {
		if (privilege == null)
			throw new MalformedPrivilege("privilege cannot be null.");
		else if (obj == null)
			throw new MalformedPrivilege("type cannot be null.");
		else if (privilege == PRIVILEGE.CREATE_VIEW)
			throw new MalformedPrivilege("CREATE VIEW is not an On-privilege.");
		else {
			this.privilege = privilege;
			this.obj = obj;
		}
	}

	/* (non-Javadoc)
	 * @see guarnieri.marco.ethz.privileges.IPrivilege#toSql()
	 */
	@Override
	public String toSql() {
		return privilege.toString() + " " + obj.getId();
	}

	/**
	 * The object.
	 * 
	 * @return The object.
	 */
	public IType getObj() {
		return obj;
	}

	/**
	 * Get the privilege.
	 * 
	 * @return The privilege.
	 */
	public PRIVILEGE getPrivilege() {
		return privilege;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return this.toSql();
	}
}
