package prototype.privileges;

import prototype.interfaces.CopyException;
import prototype.objects.IType;

/**
 * Class representing the SELECT ON TABLE privilege.
 * 
 * 
 *
 */
public class SelectPrivilege extends AbstractPrivilege {

	/**
	 * Constructor.
	 * 
	 * @param obj
	 *            - The object on which we have the privilege.
	 * @throws MalformedPrivilege
	 */
	public SelectPrivilege(IType obj) throws MalformedPrivilege {
		super(PRIVILEGE.SELECT, obj);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.privileges.IPrivilege#copy()
	 */
	@Override
	public IPrivilege copy() throws CopyException {
		try {
			return new SelectPrivilege(getObj());
		} catch (MalformedPrivilege e) {
			throw new CopyException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		else if (obj instanceof SelectPrivilege) {
			if (obj == this)
				return true;
			else
				return ((SelectPrivilege) obj).getPrivilege().equals(this.getPrivilege()) && ((SelectPrivilege) obj).getObj().equals(this.getObj());
		} else
			return false;

	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 31 * result + getPrivilege().hashCode();
		result = 31 * result + getObj().hashCode();
		return result;
	}
}
