package prototype.queries.domain.relational.calculus.syntax.expressions;

import prototype.queries.MalformedExpressionException;
import prototype.queries.domain.relational.calculus.syntax.IOperand;
import prototype.queries.domain.relational.calculus.visitors.IVisitor;

/**
 * Class representing constants.
 * 
 * 
 *
 */
public class Constant implements IOperand {

	/**
	 * Constant value identifier.
	 */
	private String value;

	/**
	 * Constructor.
	 * 
	 * @param value
	 *            - The constant value.
	 * @throws MalformedExpressionException
	 */
	public Constant(String value) throws MalformedExpressionException {
		if (value == null || value.isEmpty())
			throw new MalformedExpressionException("Value cannot be null or empty.");
		else
			this.value = value;
	}

	/**
	 * The value.
	 * 
	 * @return the value.
	 */
	public String getValue() {
		return value;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "\'" + this.getValue() + "\'";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.queries.visitors.IDrcVisitable#accept(guarnieri.marco.ethz.queries.visitors.IDrcVisitor)
	 */
	@Override
	public void accept(IVisitor visitor) {
		visitor.visit(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		else if (obj instanceof Constant) {
			if (obj == this)
				return true;
			else
				return ((Constant) obj).getValue().equals(this.getValue());
		} else
			return false;
	}
	
	@Override
	public Constant copy() throws MalformedExpressionException {
		return new Constant(value);
	}
	
	@Override
	public int hashCode() {
		int result = 17;
		result = 31 * result + getValue().hashCode();
		return result;
	}
}
