package prototype.queries.domain.relational.calculus.syntax.expressions;

import prototype.queries.MalformedExpressionException;
import prototype.queries.domain.relational.calculus.syntax.IExpression;
import prototype.queries.domain.relational.calculus.syntax.IOperand;
import prototype.queries.domain.relational.calculus.visitors.IVisitor;
import prototype.queries.operators.Equality;

/**
 * Class representing expressions of the form op1 = op2
 * 
 * 
 *
 */
/**
 * 
 *
 */
public class EqualityExpression implements IExpression {

	/**
	 * First operand.
	 */
	private IOperand op1;
	/**
	 * The equality operator
	 */
	private Equality op;
	/**
	 * Second operand.
	 */
	private IOperand op2;

	/**
	 * Constructor
	 * 
	 * @param op1
	 *            - The first operand.
	 * @param op2
	 *            - The second operand.
	 * @throws MalformedExpressionException
	 */
	public EqualityExpression(IOperand op1, IOperand op2) throws MalformedExpressionException {
		if (op1 == null)
			throw new MalformedExpressionException("Op1 cannot be null.");
		else if (op2 == null)
			throw new MalformedExpressionException("Op2 cannot be null.");
		else {
			this.op1 = op1;
			this.op = Equality.getInstance();
			this.op2 = op2;
		}
	}

	/**
	 * Get the operator.
	 * 
	 * @return The operator.
	 */
	public Equality getOp() {
		return op;
	}

	/**
	 * Get the first operand.
	 * 
	 * @return The first operand.
	 */
	public IOperand getOp1() {
		return op1;
	}

	/**
	 * Get the second operand.
	 * 
	 * @return The second operand.
	 */
	public IOperand getOp2() {
		return op2;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {

		return op1 + " " + op + " " + op2;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.queries.visitors.IDrcVisitable#accept(guarnieri.marco.ethz.queries.visitors.IDrcVisitor)
	 */
	@Override
	public void accept(IVisitor visitor) {
		visitor.visit(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		else if (obj instanceof EqualityExpression) {
			if (obj == this)
				return true;
			else
				return ((EqualityExpression) obj).getOp1().equals(this.getOp1()) && ((EqualityExpression) obj).getOp2().equals(this.getOp2())
						&& ((EqualityExpression) obj).getOp().equals(this.getOp());
		} else
			return false;
	}
	
	@Override
	public EqualityExpression copy() throws MalformedExpressionException {
		return new EqualityExpression(op1.copy(), op2.copy());
	}
	
	@Override
	public int hashCode() {
		int result = 17;
		result = 31 * result + getOp1().hashCode();
		result = 31 * result + getOp2().hashCode();
		result = 31 * result + getOp().hashCode();
		return result;
	}
}
