package prototype.queries.domain.relational.calculus.syntax.expressions;

import java.util.List;
import java.util.Vector;

import prototype.objects.IType;
import prototype.queries.MalformedExpressionException;
import prototype.queries.domain.relational.calculus.syntax.IExpression;
import prototype.queries.domain.relational.calculus.syntax.IOperand;
import prototype.queries.domain.relational.calculus.visitors.IVisitor;
import prototype.util.Util;

/**
 * Class representing a predicates
 * 
 * 
 *
 */
public class PredicateExpression implements IExpression {

	/**
	 * The predicate's type.
	 */
	private IType type;
	/**
	 * The predicate's operands.
	 */
	private List<IOperand> operands;

	/**
	 * Constructor.
	 * 
	 * @param type
	 *            - The type
	 * @param variables
	 *            - The variables
	 * @throws MalformedExpressionException
	 */
	public PredicateExpression(IType type, List<IOperand> operands) throws MalformedExpressionException {
		if (type == null)
			throw new MalformedExpressionException("Type cannot be null.");
		else if (operands == null || operands.isEmpty())
			throw new MalformedExpressionException("Operands cannot be null or empty.");
		else if (type.getCardinality() != operands.size())
			throw new MalformedExpressionException("The number of operands is different from type's cardinality.");
		else {
			this.type = type;
			this.operands = operands;
		}
	}

	/**
	 * Get the type.
	 * 
	 * @return The type.
	 */
	public IType getType() {
		return type;
	}

	/**
	 * Get the operands.
	 * 
	 * @return The operands.
	 */
	public List<IOperand> getOperands() {
		return operands;
	}
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String list = "";
		for(IOperand o : operands)
			list += o + ", ";
		list = Util.replaceLast(list, ", ", "");
		return type.getId() + " (" + list + ")";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.queries.visitors.IDrcVisitable#accept(guarnieri.marco.ethz.queries.visitors.IDrcVisitor)
	 */
	@Override
	public void accept(IVisitor visitor) {
		visitor.visit(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		else if (obj instanceof PredicateExpression) {
			if (obj == this)
				return true;
			else
				return ((PredicateExpression) obj).getType().equals(this.getType())
						&& ((PredicateExpression) obj).getOperands().equals(this.getOperands());
		} else
			return false;
	}
	
	@Override
	public int hashCode() {
		int result = 17;
		result = 31 * result + getType().hashCode();
		result = 31 * result + getOperands().hashCode();
		return result;
	}
	
	@Override
	public PredicateExpression copy() throws MalformedExpressionException {
		List<IOperand> copiedOperands = new Vector<IOperand>();
		for(IOperand op : operands)
			copiedOperands.add(op.copy());
		return new PredicateExpression(type, copiedOperands);
	}

}
