package prototype.queries.domain.relational.calculus.syntax.expressions;

import prototype.queries.MalformedExpressionException;
import prototype.queries.domain.relational.calculus.syntax.IExpression;
import prototype.queries.domain.relational.calculus.visitors.IVisitor;
import prototype.queries.operators.NotOperator;
import prototype.queries.operators.Operator;

/**
 * Class representing a binary expression of the form operator expr
 * 
 * 
 *
 */
public class UnaryExpression implements IExpression {

	/**
	 * The expression.
	 */
	private IExpression expr;

	/**
	 * The binary operator.
	 */
	private Operator op;

	/**
	 * Constructor.
	 * 
	 * @param op
	 *            - Unary operator
	 * @param expr
	 *            - The expression
	 * @throws MalformedExpressionException
	 */
	public UnaryExpression(Operator op, IExpression expr) throws MalformedExpressionException {
		if (expr == null)
			throw new MalformedExpressionException("Expr cannot be null.");
		else if (op == null)
			throw new MalformedExpressionException("Op cannot be null.");
		else if (op.getMaxArity() != 1)
			throw new MalformedExpressionException("Op must be a unary operator.");
		else if (!(op instanceof NotOperator))
			throw new MalformedExpressionException("Op must be the NOT operator.");
		else {
			this.op = op;
			this.expr = expr;
		}
	}

	/**
	 * Get the expression.
	 * 
	 * @return The expression.
	 */
	public IExpression getExpression() {
		return expr;
	}

	/**
	 * Get the operator.
	 * 
	 * @return The operator.
	 */
	public Operator getOperator() {
		return op;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return op + " (" + expr + ")";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.queries.visitors.IDrcVisitable#accept(guarnieri.marco.ethz.queries.visitors.IDrcVisitor)
	 */
	@Override
	public void accept(IVisitor visitor) {
		visitor.visit(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		else if (obj instanceof UnaryExpression) {
			if (obj == this)
				return true;
			else
				return ((UnaryExpression) obj).getOperator().equals(this.getOperator())
						&& ((UnaryExpression) obj).getExpression().equals(this.getExpression());
		} else
			return false;
	}
	
	@Override
	public int hashCode() {
		int result = 17;
		result = 31 * result + getExpression().hashCode();
		result = 31 * result + getOperator().hashCode();
		return result;
	}

	
	@Override
	public UnaryExpression copy() throws MalformedExpressionException {
		return new UnaryExpression(op, expr.copy());
	}
}
