package prototype.queries.domain.relational.calculus.syntax.expressions;

import prototype.queries.MalformedExpressionException;
import prototype.queries.domain.relational.calculus.syntax.IOperand;
import prototype.queries.domain.relational.calculus.visitors.IVisitor;

/**
 * Class representing variables.
 * 
 * 
 *
 */
public class Variable implements IOperand, Comparable<Variable> {

	/**
	 * Variable identifier.
	 */
	private String identifier;

	/**
	 * Constructor.
	 * 
	 * @param id
	 *            - The variable identifier.
	 * @throws MalformedExpressionException
	 */
	public Variable(String id) throws MalformedExpressionException {
		if (id == null || id.isEmpty())
			throw new MalformedExpressionException("Id cannot be null or empty.");
		else {
			this.identifier = id;
		}
	}

	/**
	 * The identifier.
	 * 
	 * @return the identifier.
	 */
	public String getIdentifier() {
		return identifier;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return this.identifier;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.queries.visitors.IDrcVisitable#accept(guarnieri.marco.ethz.queries.visitors.IDrcVisitor)
	 */
	@Override
	public void accept(IVisitor visitor) {
		visitor.visit(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		else if (obj instanceof Variable) {
			if (obj == this)
				return true;
			else
				return ((Variable) obj).getIdentifier().equals(this.getIdentifier());
		} else
			return false;
	}

	@Override
	public IOperand copy() throws MalformedExpressionException {
		return new Variable(getIdentifier());
	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 31 * result + getIdentifier().hashCode();
		return result;
	}

	@Override
	public int compareTo(Variable o) {
		return this.identifier.compareTo(o.identifier);
	}

}
