package prototype.queries.domain.relational.calculus.visitors;

import java.util.HashSet;
import java.util.Set;

import prototype.queries.domain.relational.calculus.syntax.IExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.Constant;
import prototype.queries.domain.relational.calculus.syntax.expressions.EqualityExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.FalseExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.PolyadicExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.PredicateExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.QuantifiedExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.TrueExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.UnaryExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.Variable;
import prototype.queries.operators.AndOperator;
import prototype.queries.operators.Equality;
import prototype.queries.operators.ExistentialQuantifier;
import prototype.queries.operators.NotOperator;
import prototype.queries.operators.OrOperator;
import prototype.queries.operators.UniversalQuantifier;

/**
 * Checks whether the expression uses just unique identifiers, i.e., (1) there are no two variables with the same identifier but different type, and
 * (2) no identifier is instantiated multiple times.
 * 
 * 
 *
 */
public class HasUniqueIdentifiersVisitor implements IVisitor {

	/**
	 * The set of quantified variables.
	 */
	private Set<Variable> variables;

	/**
	 * The result.
	 */
	private boolean result;

	public HasUniqueIdentifiersVisitor() {
		variables = new HashSet<Variable>();
		result = true;
	}
	
	public HasUniqueIdentifiersVisitor(Set<Variable> variables) {
		this.variables = variables;
		result = true;
	}

	@Override
	public void visit(PredicateExpression expr) {
		// if (result == true) {
		// for (IOperand o : expr.getOperands()) {
		// HasUniqueIdentifiersVisitor v = new HasUniqueIdentifiersVisitor();
		//
		// o.accept(v);
		//
		//
		//
		// this.allVariables.addAll(v.getVariables());
		// }
		// }
	}

	@Override
	public void visit(PolyadicExpression expr) {
		if (result == true) {
			for (IExpression e : expr.getExpressions()) {
				HasUniqueIdentifiersVisitor v = new HasUniqueIdentifiersVisitor();

				e.accept(v);

				result &= v.hasUniqueIdentifiers();

				// check whether a variable has been already instantiated in a previous sub-expression
				if (result == true) {

					for (Variable var : v.getVariables())
						if (this.variables.contains(var)) {
							result = false;
							break;
						}
				}

				this.variables.addAll(v.getVariables());
			}

		}
	}

	@Override
	public void visit(EqualityExpression expr) {
		// if (result == true) {
		// HasUniqueIdentifiersVisitor v1 = new HasUniqueIdentifiersVisitor();
		// HasUniqueIdentifiersVisitor v2 = new HasUniqueIdentifiersVisitor();
		//
		// expr.getOp1().accept(v1);
		// expr.getOp2().accept(v2);
		//
		// // result &= v1.hasUniqueIdentifiers();
		// // result &= v2.hasUniqueIdentifiers();
		//
		// this.allVariables.addAll(v1.getVariables());
		// this.allVariables.addAll(v2.getVariables());
		//
		// }
	}
	
	@Override
	public void visit(FalseExpression expr) {
		// do nothing
	}
	
	@Override
	public void visit(TrueExpression expr) {
		// do nothing
	}

	@Override
	public void visit(QuantifiedExpression expr) {
		if (result == true) {
			HasUniqueIdentifiersVisitor v = new HasUniqueIdentifiersVisitor();

			expr.getExpression().accept(v);

			result &= v.hasUniqueIdentifiers();

			if (result == true) {

				for (Variable var : v.getVariables())
					for (Variable var1 : expr.getVariables())
						if (var.equals(var1)) {
							result = false;
							break;
						}

				this.variables.addAll(v.getVariables());
				this.variables.addAll(expr.getVariables());

			}
		}
	}

	@Override
	public void visit(UnaryExpression expr) {
		if (result == true) {
			HasUniqueIdentifiersVisitor v = new HasUniqueIdentifiersVisitor();

			expr.getExpression().accept(v);

			result &= v.hasUniqueIdentifiers();

			this.variables.addAll(v.getVariables());

		}
	}

	@Override
	public void visit(Variable var) {
		// this.allVariables.add(var);
	}

	@Override
	public void visit(Constant expr) {
		// do nothing
	}

	@Override
	public void visit(AndOperator op) {
		// do nothing
	}

	@Override
	public void visit(Equality op) {
		// do nothing
	}

	@Override
	public void visit(ExistentialQuantifier op) {
		// do nothing
	}

	@Override
	public void visit(NotOperator op) {
		// do nothing
	}

	@Override
	public void visit(OrOperator op) {
		// do nothing
	}

	@Override
	public void visit(UniversalQuantifier op) {
		// do nothing
	}

	/**
	 * Get the set of variables used in the expression.
	 * 
	 * @return The set of variables used in the expression if this.isUnique() == true. The empty set otherwise.
	 */
	public Set<Variable> getVariables() {
		return variables;
	}

	/**
	 * Retrive the result.
	 * 
	 * @return True if the visited expression contained just unique variables. False otherwise.
	 */
	public boolean hasUniqueIdentifiers() {
		return result;
	}

}
