package prototype.queries.domain.relational.calculus.visitors;

import prototype.queries.domain.relational.calculus.syntax.expressions.Constant;
import prototype.queries.domain.relational.calculus.syntax.expressions.EqualityExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.FalseExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.PolyadicExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.PredicateExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.QuantifiedExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.TrueExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.UnaryExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.Variable;
import prototype.queries.operators.AndOperator;
import prototype.queries.operators.Equality;
import prototype.queries.operators.ExistentialQuantifier;
import prototype.queries.operators.NotOperator;
import prototype.queries.operators.OrOperator;
import prototype.queries.operators.UniversalQuantifier;

/**
 * Interface for visitors over TRC expressions.
 * 
 * 
 *
 */
public interface IVisitor {

	/**
	 * Visit the predicate expression.
	 * 
	 * @param expr
	 *            - expression to be visited.
	 */
	public void visit(PredicateExpression expr);

	/**
	 * Visit the equality expression.
	 * 
	 * @param expr
	 *            - expression to be visited.
	 */
	public void visit(EqualityExpression expr);

	/**
	 * Visit the quantified expression.
	 * 
	 * @param expr
	 *            - expression to be visited.
	 */
	public void visit(QuantifiedExpression expr);

	/**
	 * Visit the unary expression.
	 * 
	 * @param expr
	 *            - expression to be visited.
	 */
	public void visit(UnaryExpression expr);

	/**
	 * Visit the variable.
	 * 
	 * @param var
	 *            - variable to be visited.
	 */
	public void visit(Variable var);

	/**
	 * Visit the constant operand.
	 * 
	 * @param expr
	 *            - expr to be visited.
	 */
	public void visit(Constant expr);

	/**
	 * Visit the AND operator.
	 * 
	 * @param op
	 *            - The operator.
	 */
	public void visit(AndOperator op);

	/**
	 * Visit the = operator.
	 * 
	 * @param op
	 *            - The operator.
	 */
	public void visit(Equality op);

	/**
	 * Visit the EXISTS operator.
	 * 
	 * @param op
	 *            - The operator.
	 */
	public void visit(ExistentialQuantifier op);

	/**
	 * Visit the NOT operator.
	 * 
	 * @param op
	 *            - The operator.
	 */
	public void visit(NotOperator op);

	/**
	 * Visit the OR operator.
	 * 
	 * @param op
	 *            - The operator.
	 */
	public void visit(OrOperator op);

	/**
	 * Visit the FORALL operator.
	 * 
	 * @param op
	 *            - The operator.
	 */
	public void visit(UniversalQuantifier op);

	/**
	 * Visit a polyadic Expression.
	 * 
	 * @param polyadicExpression
	 *            - the expression.
	 */
	public void visit(PolyadicExpression expr);

	/**
	 * Visit a false expression.
	 * 
	 * @param expr
	 *            - the expression.
	 */
	public void visit(FalseExpression expr);
	
	/**
	 * Visit a true expression.
	 * 
	 * @param expr
	 *            - the expression.
	 */
	public void visit(TrueExpression expr);

}
