package prototype.queries.domain.relational.calculus.visitors;

import java.util.List;
import java.util.Vector;

import prototype.objects.IType;
import prototype.queries.domain.relational.calculus.syntax.IExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.Constant;
import prototype.queries.domain.relational.calculus.syntax.expressions.EqualityExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.FalseExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.PolyadicExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.PredicateExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.QuantifiedExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.TrueExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.UnaryExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.Variable;
import prototype.queries.operators.AndOperator;
import prototype.queries.operators.Equality;
import prototype.queries.operators.ExistentialQuantifier;
import prototype.queries.operators.NotOperator;
import prototype.queries.operators.OrOperator;
import prototype.queries.operators.UniversalQuantifier;

/**
 * Visitor that extracts the types directly used in the expression (note that it does not extract types from direct sub-expressions of the form
 * \exists x. \phi). Works only for SRNF formulae with unique quantified variables.
 * 
 * 
 *
 */
public class MySqlGetTypes implements IVisitor {

	/**
	 * List of all types
	 */
	private List<IType> types;

	public MySqlGetTypes() {
		this.types = new Vector<IType>();
	}

	private MySqlGetTypes(List<IType> types) {
		this.types = types;
	}

	@Override
	public void visit(PredicateExpression expr) {
		// add to types
		this.types.add(expr.getType());
	}

	

	@Override
	public void visit(PolyadicExpression expr) {

		for (IExpression e : expr.getExpressions()) {
			MySqlGetTypes v = new MySqlGetTypes(this.types);
			e.accept(v);

		}

	}

	@Override
	public void visit(EqualityExpression expr) {
		// do nothing
	}
	
	@Override
	public void visit(TrueExpression expr) {
		// do nothing
	}
	
	@Override
	public void visit(FalseExpression expr) {
		// do nothing
	}

	@Override
	public void visit(QuantifiedExpression expr) {
		if (expr.getQuantifier() instanceof ExistentialQuantifier) {

			MySqlGetTypes v = new MySqlGetTypes(new Vector<IType>());
			expr.getExpression().accept(v);

		} else if (expr.getQuantifier() instanceof UniversalQuantifier) {
			assert (false);// unreachable for Safe-Range Normal Form expressions
		} else {
			assert (false);// unreachable
		}

	}

	@Override
	public void visit(UnaryExpression expr) {
		MySqlGetTypes v = new MySqlGetTypes(this.types);
		expr.getExpression().accept(v);

	}

	@Override
	public void visit(Variable var) {
		assert (false); // unreachable
	}

	@Override
	public void visit(Constant expr) {
		assert (false); // unreachable
	}

	@Override
	public void visit(AndOperator op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(Equality op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(ExistentialQuantifier op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(NotOperator op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(OrOperator op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(UniversalQuantifier op) {
		assert (false); // unreachable
	}

	/**
	 * Get all the types.
	 * 
	 * @return The types.
	 */
	public List<IType> getTypes() {
		return types;
	}

}
