package prototype.queries.operators;

import prototype.queries.MalformedExpressionException;
import prototype.queries.domain.relational.calculus.visitors.IVisitable;

/**
 * Class representing an operator (either unary or binary).
 * 
 * 
 *
 */
public abstract class Operator implements IVisitable {

	/**
	 * The operator's name.
	 */
	private String name;

	/**
	 * The operator's maximum arity.
	 */
	private int maxArity;
	
	/**
	 * The operator's minimum arity.
	 */
	private int minArity;

	/**
	 * Constructor.
	 * 
	 * @param name
	 *            - Operator's name.
	 * @param maxArity
	 *            - Operator's maximum arity.
	 * @param minArity
	 *            - Operator's minimum arity.
	 */
	public Operator(String name, int maxArity, int minArity) throws MalformedExpressionException {
		if (name == null)
			throw new MalformedExpressionException("Name cannot be null.");
		else if (minArity != 1 && minArity != 2)
			throw new MalformedExpressionException("minArity can be only 1 or 2.");
		else if (minArity > maxArity)
			throw new MalformedExpressionException("minArity must be less or equal to maxArity.");
		else {
			this.name = name;
			this.maxArity = maxArity;
			this.minArity = minArity;
		}
	}

	/**
	 * Get the operator's maximum arity.
	 * 
	 * @return The operator's maximum arity.
	 */
	public int getMaxArity() {
		return maxArity;
	}
	
	/**
	 * Get the operator's minimum arity.
	 * 
	 * @return The operator's minimum arity.
	 */
	public int getMinArity() {
		return minArity;
	}

	/**
	 * Get the operator's name.
	 * 
	 * @return The operator's name.
	 */
	public String getName() {
		return name;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return name;
	}
}
