package prototype.queries.operators;

import prototype.queries.MalformedExpressionException;
import prototype.queries.domain.relational.calculus.visitors.IVisitor;

/**
 * Singleton class for the OR operator.
 * 
 * 
 *
 */
public class OrOperator extends Operator {
	/** The unique instance of the singleton. */
	private static OrOperator singleton_ = null;

	/**
	 * Get the singleton instance of the operator.
	 * 
	 * @return The operator.
	 * @throws MalformedExpressionException
	 */
	public static OrOperator getInstance() throws MalformedExpressionException {
		if (singleton_ == null) {
			singleton_ = new OrOperator("OR", Integer.MAX_VALUE, 2);
		}
		return singleton_;
	}

	/**
	 * Private Constructor.
	 * 
	 * @param name
	 *            - Operator's name.
	 * @param maxArity
	 *            - Operator's maximum arity.
	 * @param minArity
	 *            - Operator's minimum arity.
	 */
	private OrOperator(String name, int maxArity, int minArity) throws MalformedExpressionException {
		super(name, maxArity, minArity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.queries.visitor.IVisitable#accept(guarnieri.marco.ethz.queries.visitor.IVisitor)
	 */
	@Override
	public void accept(IVisitor visitor) {
		visitor.visit(this);
	}
}
