package prototype.queries.translation;

import java.util.LinkedHashSet;
import java.util.Set;

import prototype.queries.domain.relational.calculus.syntax.IExpression;
import prototype.queries.domain.relational.calculus.syntax.IOperand;
import prototype.queries.domain.relational.calculus.syntax.expressions.Constant;
import prototype.queries.domain.relational.calculus.syntax.expressions.EqualityExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.FalseExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.PolyadicExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.PredicateExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.QuantifiedExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.TrueExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.UnaryExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.Variable;
import prototype.queries.domain.relational.calculus.visitors.IVisitor;
import prototype.queries.operators.AndOperator;
import prototype.queries.operators.Equality;
import prototype.queries.operators.ExistentialQuantifier;
import prototype.queries.operators.NotOperator;
import prototype.queries.operators.OrOperator;
import prototype.queries.operators.UniversalQuantifier;

public class GetConstants implements IVisitor {

	private Set<Constant> constants;

	public GetConstants() {
		constants = new LinkedHashSet<Constant>();
	}

	private GetConstants(Set<Constant> constants) {
		this.constants = constants;
	}

	@Override
	public void visit(PredicateExpression expr) {
		for (IOperand op : expr.getOperands())
			if (op instanceof Constant)
				constants.add((Constant) op);
	}

	@Override
	public void visit(EqualityExpression expr) {
		if (expr.getOp1() instanceof Constant)
			constants.add((Constant) expr.getOp1());
		if (expr.getOp2() instanceof Constant)
			constants.add((Constant) expr.getOp2());
	}

	@Override
	public void visit(QuantifiedExpression expr) {
		GetConstants v = new GetConstants(constants);
		expr.getExpression().accept(v);
	}

	@Override
	public void visit(UnaryExpression expr) {
		GetConstants v = new GetConstants(constants);
		expr.getExpression().accept(v);
	}

	@Override
	public void visit(Variable var) {
		assert (false); // unreachable
	}

	@Override
	public void visit(Constant expr) {
		assert (false); // unreachable
	}

	@Override
	public void visit(AndOperator op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(Equality op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(ExistentialQuantifier op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(NotOperator op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(OrOperator op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(UniversalQuantifier op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(PolyadicExpression expr) {
		for(IExpression e : expr.getExpressions())
		{
			GetConstants v = new GetConstants(constants);
			e.accept(v);
		}
	}

	@Override
	public void visit(FalseExpression expr) {
		// do nothing
	}

	@Override
	public void visit(TrueExpression expr) {
		// do nothing
	}
	
	public Set<Constant> getConstants() {
		return constants;
	}

}
