package prototype.queries.translation;

import java.util.LinkedHashSet;
import java.util.Set;

import prototype.objects.IType;
import prototype.queries.domain.relational.calculus.syntax.IExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.Constant;
import prototype.queries.domain.relational.calculus.syntax.expressions.EqualityExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.FalseExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.PolyadicExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.PredicateExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.QuantifiedExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.TrueExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.UnaryExpression;
import prototype.queries.domain.relational.calculus.syntax.expressions.Variable;
import prototype.queries.domain.relational.calculus.visitors.IVisitor;
import prototype.queries.operators.AndOperator;
import prototype.queries.operators.Equality;
import prototype.queries.operators.ExistentialQuantifier;
import prototype.queries.operators.NotOperator;
import prototype.queries.operators.OrOperator;
import prototype.queries.operators.UniversalQuantifier;

public class GetTypes implements IVisitor {

	private Set<IType> types;

	public GetTypes() {
		types = new LinkedHashSet<IType>();
	}

	private GetTypes(Set<IType> types) {
		this.types = types;
	}

	@Override
	public void visit(PredicateExpression expr) {
		this.types.add(expr.getType());
	}

	@Override
	public void visit(EqualityExpression expr) {
		// do nothing
	}

	@Override
	public void visit(QuantifiedExpression expr) {
		GetTypes v = new GetTypes(types);
		expr.getExpression().accept(v);
	}

	@Override
	public void visit(UnaryExpression expr) {
		GetTypes v = new GetTypes(types);
		expr.getExpression().accept(v);
	}

	@Override
	public void visit(Variable var) {
		assert (false); // unreachable
	}

	@Override
	public void visit(Constant expr) {
		assert (false); // unreachable
	}

	@Override
	public void visit(AndOperator op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(Equality op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(ExistentialQuantifier op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(NotOperator op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(OrOperator op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(UniversalQuantifier op) {
		assert (false); // unreachable
	}

	@Override
	public void visit(PolyadicExpression expr) {
		for(IExpression e : expr.getExpressions())
		{
			GetTypes v = new GetTypes(types);
			e.accept(v);
		}
	}

	@Override
	public void visit(FalseExpression expr) {
		// do nothing
	}

	@Override
	public void visit(TrueExpression expr) {
		// do nothing
	}
	
	public Set<IType> getTypes() {
		return types;
	}

}
