package prototype.results;

import prototype.results.accessControlResult.IAccessControlResult;
import prototype.results.actionResults.IActionResult;

/**
 * Public interface for results. A result contains information about a statement's or object's execution. It contains the actual statement's result
 * together with the access control result, i.e., whether the execution has been authorized by the access control mechanism.
 * 
 * 
 *
 */
public interface IResult {

	/**
	 * Get the action's result.
	 * 
	 * @return The action's result.
	 */
	public IActionResult getActionResult();

	/**
	 * Get the access control decision.
	 * 
	 * @return The access control decision.
	 */
	public IAccessControlResult getAccessControlDecision();

}
