package prototype.results.implementation;

import prototype.results.actionResults.IBooleanActionResult;
import prototype.results.explanation.IIntegrityExplanation;

/**
 * Class representing the Single action result.
 * 
 * 
 *
 */
public class BooleanActionResult implements IBooleanActionResult {

	/**
	 * Flag denoting whether the action has caused an exception or not.
	 */
	private boolean exception;
	/**
	 * The explanation.
	 */
	private IIntegrityExplanation explanation;
	/**
	 * The action's result.
	 */
	private boolean result;

	/**
	 * Constructor.
	 * 
	 * @param result
	 *            - The action's result.
	 * @param exception
	 *            - The exception flag.
	 * @param explanation
	 *            - The explanation.
	 * @throws MalformedResultException
	 */
	public BooleanActionResult(boolean result, boolean exception, IIntegrityExplanation explanation) throws MalformedResultException {

		if ((exception == true && explanation != null) || (exception == false && explanation == null)) {
			this.result = result;
			this.exception = exception;
			this.explanation = explanation;
		} else
			throw new MalformedResultException("Exception and explanation do not match.");
	}

	@Override
	public boolean exception() {
		return this.exception;
	}

	@Override
	public IIntegrityExplanation getExceptionExplanation() {
		return this.explanation;
	}

	@Override
	public boolean getResult() {
		return this.result;
	}
	
	@Override
	public String toString() {
		return "Result: "+result+" Exception: "+exception + (exception ? " Explanation: "+explanation.toString() : "");
	}

}
