package prototype.results.implementation;

import java.util.Set;

import prototype.queries.domain.relational.calculus.syntax.IOperand;
import prototype.queries.domain.relational.calculus.syntax.expressions.Variable;
import prototype.results.actionResults.INonBooleanActionResult;
import prototype.results.explanation.IIntegrityExplanation;
import prototype.tuple.ITuple;

/**
 * Class representing the Single action result.
 * 
 * 
 *
 */
public class NonBooleanActionResult implements INonBooleanActionResult {

	/**
	 * Flag denoting whether the action has caused an exception or not.
	 */
	private boolean exception;
	/**
	 * The explanation.
	 */
	private IIntegrityExplanation explanation;
	/**
	 * The action's result.
	 */
	private Set<ITuple> result;

	/**
	 * Constructor.
	 * 
	 * @param result
	 *            - The action's result.
	 * @param exception
	 *            - The exception flag.
	 * @param explanation
	 *            - The explanation.
	 * @throws MalformedResultException
	 */
	public NonBooleanActionResult(Set<ITuple> result, boolean exception, IIntegrityExplanation explanation) throws MalformedResultException {

		if ((exception == true && explanation != null) || (exception == false && explanation == null)) {

			if (result != null) {

				for (ITuple t : result)
					for (IOperand o : t.getContent())
						if (o instanceof Variable)
							throw new MalformedResultException("Result cannot contain free variables.");

				this.result = result;
				this.exception = exception;
				this.explanation = explanation;
			} else
				throw new MalformedResultException("Result cannot be null.");

		} else
			throw new MalformedResultException("Exception and explanation do not match.");
	}

	@Override
	public boolean exception() {
		return this.exception;
	}

	@Override
	public IIntegrityExplanation getExceptionExplanation() {
		return this.explanation;
	}

	@Override
	public Set<ITuple> getResult() {
		return this.result;
	}
	
	@Override
	public String toString() {
		return "Result is Empty: "+result.isEmpty()+" Exception: "+exception + (exception ? " Explanation: "+explanation.toString() : "");
	}

}
