package prototype.results.implementation;

import prototype.results.IResult;
import prototype.results.accessControlResult.IAccessControlResult;
import prototype.results.actionResults.IActionResult;

/**
 * Class representing results.
 * 
 * 
 *
 */
public class Result implements IResult {

	/**
	 * The action's result.
	 */
	private IActionResult actionResult;

	/**
	 * The access control result.
	 */
	private IAccessControlResult accessControlResult;

	/**
	 * Constructor.
	 * 
	 * @param actionResult
	 *            - The action result.
	 * @param accessControlResult
	 *            - The access control result.
	 * @throws MalformedResultException
	 */
	public Result(IActionResult actionResult, IAccessControlResult accessControlResult) throws MalformedResultException {
		if (accessControlResult != null) {
			this.actionResult = actionResult;
			this.accessControlResult = accessControlResult;
		} else
			throw new MalformedResultException("AccessControlResult cannot be null.");
	}

	@Override
	public IActionResult getActionResult() {
		return this.actionResult;
	}

	@Override
	public IAccessControlResult getAccessControlDecision() {
		return this.accessControlResult;
	}

	@Override
	public String toString() {
		return "ACCESS CONTROL RESULT" + System.getProperty("line.separator") + "\t\t" + this.accessControlResult
				+ System.getProperty("line.separator")
				+ (actionResult == null ? "" : "ACTION RESULT" + System.getProperty("line.separator") + "\t\t" + actionResult.toString());
	}
}
