package prototype.results.implementation;

import prototype.results.accessControlResult.ITriggerAccessControlResult;
import prototype.results.explanation.ISecurityExplanation;

/**
 * Class representing access control results for triggers.
 * 
 * 
 *
 */
public class TriggerAccessControlResult implements ITriggerAccessControlResult {

	/**
	 * The access control result associated with the trigger's condition.
	 */
	private ActionAccessControlResult condition;
	/**
	 * The access control result associated with the trigger's action.
	 */
	private ActionAccessControlResult action;

	/**
	 * Constructor.
	 * 
	 * @param conditionResult
	 *            - the access control decision for the trigger's condition.
	 * @param conditionExplanation
	 *            - the explanation for the trigger's condition.
	 * @param actionResult
	 *            - the access control decision for the trigger's action.
	 * @param actionExplanation
	 *            - the explanation for the trigger's action.
	 * @throws MalformedResultException
	 */
	public TriggerAccessControlResult(boolean conditionResult, ISecurityExplanation conditionExplanation, boolean actionResult,
			ISecurityExplanation actionExplanation) throws MalformedResultException {

		this.condition = new ActionAccessControlResult(conditionResult, conditionExplanation);
		if (conditionResult)
			this.action = new ActionAccessControlResult(actionResult, actionExplanation);
		else
			this.action = null;

	}

	/**
	 * Constructor.
	 * 
	 * @param conditionResult
	 *            - the access control decision for the trigger's condition.
	 * @param conditionExplanation
	 *            - the explanation for the trigger's condition.
	 * @param actionResult
	 *            - the access control decision for the trigger's action.
	 * @param actionExplanation
	 *            - the explanation for the trigger's action.
	 * @throws MalformedResultException
	 */
	public TriggerAccessControlResult(boolean conditionResult, ISecurityExplanation conditionExplanation) throws MalformedResultException {

		this.condition = new ActionAccessControlResult(conditionResult, conditionExplanation);

		this.action = null;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.results.accessControlResult.ITriggerAccessControlResult#getTriggerConditionResult()
	 */
	@Override
	public boolean getTriggerConditionResult() {
		return this.condition.getResult();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.results.accessControlResult.ITriggerAccessControlResult#getTriggerConditionExplanation()
	 */
	@Override
	public ISecurityExplanation getTriggerConditionExplanation() {
		return this.condition.getExplanation();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.results.accessControlResult.ITriggerAccessControlResult#getTriggerActionResult()
	 */
	@Override
	public boolean getTriggerActionResult() {
		if (action != null)
			return this.action.getResult();
		else
			return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.results.accessControlResult.ITriggerAccessControlResult#getTriggerActionExplanation()
	 */
	@Override
	public ISecurityExplanation getTriggerActionExplanation() {
		if (action != null)
			return this.action.getExplanation();
		else
			return null;
	}

	@Override
	public String toString() {
		return "CONDITION: " + condition + " ACTION: " + (action == null ? "NULL" : action.toString());
	}

}
