package prototype.results.implementation;

import prototype.results.actionResults.IBooleanActionResult;
import prototype.results.actionResults.ITriggerResult;
import prototype.results.explanation.IIntegrityExplanation;

/**
 * Class representing trigger's results.
 * 
 * 
 *
 */
public class TriggerResult implements ITriggerResult {

	/**
	 * The result associated with the condition.
	 */
	private BooleanActionResult condition;

	/**
	 * The result associated with the action.
	 */
	private IBooleanActionResult action;

	/**
	 * Constructor.
	 * 
	 * @param conditionResult
	 *            - The condition's result.
	 * @param actionResult
	 *            - The action's result.
	 * @param exception
	 *            - The exception associated with the trigger's action.
	 * @param explanation
	 *            - The explanation associated with the trigger's action.
	 * @throws MalformedResultException
	 */
	public TriggerResult(boolean conditionResult, boolean actionResult, boolean exception, IIntegrityExplanation explanation)
			throws MalformedResultException {
		this.condition = new BooleanActionResult(conditionResult, false, null);
		if (conditionResult)
			this.action = new BooleanActionResult(actionResult, exception, explanation);
		else
			this.action = null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.results.actionResults.ITriggerResult#getConditionResult()
	 */
	@Override
	public boolean getConditionResult() {
		return condition.getResult();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.results.actionResults.ITriggerResult#getActionException()
	 */
	@Override
	public boolean getActionException() {
		if (action != null)
			return action.exception();
		else
			return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.results.actionResults.ITriggerResult#getExceptionExplanation()
	 */
	@Override
	public IIntegrityExplanation getExceptionExplanation() {
		if (action != null)
			return action.getExceptionExplanation();
		else
			return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.results.actionResults.ITriggerResult#getActionResult()
	 */
	@Override
	public boolean getActionResult() {
		if (action != null)
			return action.getResult();
		else
			return false;
	}

	@Override
	public String toString() {
		return "CONDITION: " + condition + " ACTION: " + (action == null ? "NULL" : action.toString());
	}

}
