package prototype.statements.abstractImplementation;

import prototype.interfaces.CopyException;
import prototype.statements.MalformedStatementException;
import prototype.statements.interfaces.IStatement;
import prototype.statements.interfaces.TranslationException;
import prototype.users.IUser;

public abstract class Statement implements IStatement {

	/**
	 * The user executing the command
	 */
	private IUser invoker;

	/**
	 * Constructor.
	 * 
	 * @param invoker
	 *            - the user executing the command.
	 * @throws MalformedStatementException
	 *             - if the invoker is null.
	 */
	public Statement(IUser invoker) throws MalformedStatementException {
		if (invoker != null)
			try {
				this.invoker = invoker.copy();
			} catch (CopyException e) {
				throw new MalformedStatementException(e);
			}
		else
			throw new MalformedStatementException("The invoker cannot be null.");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.IStatement#getInvoker()
	 */
	@Override
	public IUser getInvoker() {
		return this.invoker;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.IStatement#toSQL()
	 */
	@Override
	public abstract String toSQL() throws TranslationException;
	
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return this.getDefinition();
	}

	
	
}
