package prototype.statements.abstractImplementation.add;

import prototype.results.actionResults.IActionResult;
import prototype.statements.MalformedStatementException;
import prototype.statements.abstractImplementation.Statement;
import prototype.statements.interfaces.TranslationException;
import prototype.statements.interfaces.add.IAddUser;
import prototype.system.implementation.ExecutionException;
import prototype.system.interfaces.IDatabaseEngine;
import prototype.system.interfaces.IState;
import prototype.users.IUser;
import prototype.util.Pair;

/**
 * Abstract class for CREATE USER statements.
 * 
 * 
 *
 */
public abstract class AddUser extends Statement implements IAddUser {

	/**
	 * The user to add.
	 */
	private IUser toAdd;

	/**
	 * Constructor.
	 * 
	 * @param invoker
	 *            - The invoking user.
	 * @param toAdd
	 *            - The user to add.
	 * @throws MalformedStatementException
	 */
	public AddUser(IUser invoker, IUser toAdd) throws MalformedStatementException {
		super(invoker);
		if (toAdd != null)
			this.toAdd = toAdd;
		else
			throw new MalformedStatementException("toAdd cannot be null");
	}

	@Override
	public String getDefinition() {
		return this.getInvoker() + ", CREATE USER " + this.getUser();
	}

	@Override
	public IUser getUser() {
		return this.toAdd;
	}

	@Override
	public abstract String toSQL() throws TranslationException;

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.interfaces.IExecutable#execute(guarnieri.marco.ethz.system.interfaces.IExecutionEngine,
	 * guarnieri.marco.ethz.system.interfaces.IState)
	 */
	@Override
	public Pair<IActionResult,IState> execute(IDatabaseEngine engine, IState state)  throws ExecutionException{
		return engine.execute(this, state);
	}

	
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		else if (obj instanceof AddUser) {
			if (obj == this)
				return true;
			else
				return ((AddUser) obj).getInvoker().equals(this.getInvoker())
						&& ((AddUser) obj).getUser().equals(this.getUser());
		} else
			return false;
	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 31 * result + getInvoker().hashCode();
		result = 31 * result + getUser().hashCode();
		return result;
	}
}
