package prototype.statements.abstractImplementation.changePolicy;

import prototype.interfaces.CopyException;
import prototype.privileges.IPrivilege;
import prototype.statements.MalformedStatementException;
import prototype.statements.abstractImplementation.Statement;
import prototype.statements.interfaces.TranslationException;
import prototype.statements.interfaces.changePolicy.IChangePolicyStatement;
import prototype.statements.modes.CHANGE_POLICY_MODE;
import prototype.users.IUser;

/**
 * Abstract class for GRANT, REVOKE, and GRANT WITH GRANT OPTION statements.
 * 
 * 
 *
 */
public abstract class ChangePolicyStatement extends Statement implements IChangePolicyStatement {

	/**
	 * The statement's target.
	 */
	private IUser target;
	/**
	 * The statement's privilege.
	 */
	private IPrivilege privilege;
	/**
	 * The statement's mode.
	 */
	private CHANGE_POLICY_MODE mode;

	/**
	 * Constructor.
	 * 
	 * @param invoker
	 *            - The user executing the statement.
	 * @param mode
	 *            - The mode.
	 * @param privilege
	 *            - The privilege.
	 * @param target
	 *            - The target of the statement.
	 * @throws MalformedStatementException
	 */
	public ChangePolicyStatement(IUser invoker, CHANGE_POLICY_MODE mode, IPrivilege privilege, IUser target) throws MalformedStatementException {
		super(invoker);
		if (mode == null) {
			throw new MalformedStatementException("Mode cannot be null.");
		} else if (privilege == null) {
			throw new MalformedStatementException("Privilege cannot be null.");
		} else if (target == null) {
			throw new MalformedStatementException("Target cannot be null.");
		} else {
			this.mode = mode;
			try {
				this.privilege = privilege.copy();
				this.target = target.copy();
			} catch (CopyException e) {
				throw new MalformedStatementException(e);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.interfaces.IChangePolicyStatement#getPrivilege()
	 */
	@Override
	public IPrivilege getPrivilege() {
		return this.privilege;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.interfaces.IChangePolicyStatement#getTarget()
	 */
	@Override
	public IUser getTarget() {
		return this.target;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.interfaces.IChangePolicyStatement#getMode()
	 */
	@Override
	public CHANGE_POLICY_MODE getMode() {
		return this.mode;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.implementation.abstraction.Statement#toSQL()
	 */
	@Override
	public abstract String toSQL() throws TranslationException;

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		else if (obj instanceof ChangePolicyStatement) {
			if (obj == this)
				return true;
			else
				return ((ChangePolicyStatement) obj).getInvoker().equals(this.getInvoker())
						&& ((ChangePolicyStatement) obj).getTarget().equals(this.getTarget())
						&& ((ChangePolicyStatement) obj).getMode().equals(this.getMode())
						&& ((ChangePolicyStatement) obj).getPrivilege().equals(this.getPrivilege());
		} else
			return false;
	}
	
	@Override
	public int hashCode() {
		int result = 17;
		result = 31 * result + getInvoker().hashCode();
		result = 31 * result + getTarget().hashCode();
		result = 31 * result + getMode().hashCode();
		result = 31 * result + getPrivilege().hashCode();
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.interfaces.IStatement#getDefinition()
	 */
	@Override
	public String getDefinition() {
		return "["+this.getMode().toString() + ", " + this.getTarget() + ", " + this.getPrivilege().toString() + ", " + this.getInvoker()+"]";
	}
	
	

}
