package prototype.statements.abstractImplementation.create;

import prototype.interfaces.CopyException;
import prototype.objects.IDbObject;
import prototype.objects.ITable;
import prototype.objects.ITrigger;
import prototype.objects.IView;
import prototype.statements.MalformedStatementException;
import prototype.statements.abstractImplementation.Statement;
import prototype.statements.interfaces.TranslationException;
import prototype.statements.interfaces.create.ICreateStatement;
import prototype.statements.modes.CREATE_MODE;
import prototype.users.IUser;

/**
 * Abstract class for all CREATE statements (CREATE VIEW, CREATE TABLE, and CREATE TRIGGER).
 * 
 * 
 *
 */
public abstract class CreateStatement extends Statement implements ICreateStatement {

	/**
	 * The object created by this statement.
	 */
	private IDbObject obj;

	/**
	 * Constructor.
	 * 
	 * @param invoker
	 *            - The user executing the command.
	 * @param object
	 *            - The object to be created.
	 * @throws MalformedStatementException
	 */
	public CreateStatement(IUser invoker, IDbObject object) throws MalformedStatementException {
		super(invoker);
		if (object != null)
			try {
				this.obj = object.copy();
			} catch (CopyException e) {
				throw new MalformedStatementException(e);
			}
		else
			throw new MalformedStatementException("The object cannot be null.");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.interfaces.ICreateStatement#getObject()
	 */
	@Override
	public IDbObject getObject() {
		return this.obj;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.interfaces.ICreateStatement#getMode()
	 */
	@Override
	public CREATE_MODE getMode() {
		if (obj instanceof ITable)
			return CREATE_MODE.TABLE;
		else if (obj instanceof ITrigger)
			return CREATE_MODE.TRIGGER;
		else if (obj instanceof IView)
			return CREATE_MODE.VIEW;
		else {
			assert (false);
			return null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.implementation.Statement#toSQL()
	 */
	@Override
	public abstract String toSQL() throws TranslationException;
	
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		else if (obj instanceof CreateStatement) {
			if (obj == this)
				return true;
			else
				return ((CreateStatement) obj).getInvoker().equals(this.getInvoker())
						&& ((CreateStatement) obj).getObject().equals(this.getObject())
						&& ((CreateStatement) obj).getMode().equals(this.getMode());
		} else
			return false;
	}
	
	@Override
	public int hashCode() {
		int result = 17;
		result = 31 * result + getInvoker().hashCode();
		result = 31 * result + getObject().hashCode();
		result = 31 * result + getMode().hashCode();
		return result;
	}
	
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.interfaces.IStatement#getDefinition()
	 */
	@Override
	public String getDefinition() {
		return this.getInvoker() +", CREATE "+this.getMode().toString()+", "+this.getObject();
	}

}
