package prototype.statements.abstractImplementation.select;

import prototype.queries.domain.relational.calculus.IQuery;
import prototype.results.actionResults.IActionResult;
import prototype.statements.MalformedStatementException;
import prototype.statements.abstractImplementation.Statement;
import prototype.statements.interfaces.TranslationException;
import prototype.statements.interfaces.select.ISelectStatement;
import prototype.system.implementation.ExecutionException;
import prototype.system.interfaces.IDatabaseEngine;
import prototype.system.interfaces.IState;
import prototype.users.IUser;
import prototype.util.Pair;

/**
 * Abstract class for SELECT statements.
 * 
 * 
 *
 */
public abstract class SelectStatement extends Statement implements ISelectStatement {

	/**
	 * The query.
	 */
	private IQuery query;

	/**
	 * Constructor.
	 * 
	 * @param invoker
	 *            - the user executing the query.
	 * @param query
	 *            - the query.
	 * @throws MalformedStatementException
	 */
	public SelectStatement(IUser invoker, IQuery query) throws MalformedStatementException {
		super(invoker);
		if (query != null)
			this.query = query;
		else
			throw new MalformedStatementException("The query cannot be null");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.interfaces.ISelectStatement#getQuery()
	 */
	@Override
	public IQuery getQuery() {
		return this.query;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.implementation.Statement#toSQL()
	 */
	@Override
	public abstract String toSQL() throws TranslationException;
	
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		else if (obj instanceof SelectStatement) {
			if (obj == this)
				return true;
			else
				return ((SelectStatement) obj).getInvoker().equals(this.getInvoker())
						&& ((SelectStatement) obj).getQuery().equals(this.getQuery());
		} else
			return false;
	}
	
	@Override
	public int hashCode() {
		int result = 17;
		result = 31 * result + getInvoker().hashCode();
		result = 31 * result + getQuery().hashCode();
		return result;
	}
	
	
	/* (non-Javadoc)
	 * @see guarnieri.marco.ethz.statements.interfaces.IStatement#getDefinition()
	 */
	@Override
	public String getDefinition() {
		return this.getInvoker() +" : "+query.toString();
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.interfaces.IExecutable#execute(guarnieri.marco.ethz.system.interfaces.IExecutionEngine,
	 * guarnieri.marco.ethz.system.interfaces.IState)
	 */
	@Override
	public Pair<IActionResult,IState> execute(IDatabaseEngine engine, IState state) throws ExecutionException{
		return engine.execute(this, state);
	}
	
}
