package prototype.statements.implementation.mysql.changePolicy;

import prototype.interfaces.CopyException;
import prototype.privileges.IPrivilege;
import prototype.statements.MalformedStatementException;
import prototype.statements.abstractImplementation.changePolicy.GrantWithOptionStatement;
import prototype.statements.interfaces.TranslationException;
import prototype.statements.modes.CHANGE_POLICY_MODE;
import prototype.users.IUser;

/**
 * The concrete class representing GRANT WITH GRANT OPTION statements for the MySql database.
 * 
 * 
 *
 */
public class MySqlGrantWithOptionStatement extends GrantWithOptionStatement {

	/**
	 * Constructor.
	 * 
	 * @param invoker
	 *            - The user executing the statement.
	 * @param mode
	 *            - The mode.
	 * @param privilege
	 *            - The privilege.
	 * @param target
	 *            - The target of the statement.
	 * @throws MalformedStatementException
	 */
	public MySqlGrantWithOptionStatement(IUser invoker, CHANGE_POLICY_MODE mode, IPrivilege privilege, IUser target)
			throws MalformedStatementException {
		super(invoker, mode, privilege, target);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.implementation.abstraction.changePolicy.ChangePolicyStatement#toSQL()
	 */
	@Override
	public String toSQL() throws TranslationException {
		return "GRANT " + this.getPrivilege() + " TO " + this.getTarget().getUsername() + " WITH GRANT OPTION" + ";";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.interfaces.ICopiable#copy()
	 */
	@Override
	public MySqlGrantWithOptionStatement copy() throws CopyException {

		try {
			return new MySqlGrantWithOptionStatement(getInvoker().copy(), getMode(), (IPrivilege) getPrivilege().copy(), getTarget().copy());
		} catch (MalformedStatementException e) {
			throw new CopyException(e);
		}

	}
}
