package prototype.statements.implementation.mysql.create;

import prototype.interfaces.CopyException;
import prototype.objects.IView;
import prototype.statements.MalformedStatementException;
import prototype.statements.abstractImplementation.create.CreateViewStatement;
import prototype.statements.interfaces.TranslationException;
import prototype.statements.interfaces.create.ICreateViewStatement;
import prototype.users.IUser;

/**
 * The concrete class representing CREATE VIEW statements for the MySql database.
 * 
 * 
 *
 */
public class MySqlCreateViewStatement extends CreateViewStatement {

	/**
	 * Constructor.
	 * 
	 * @param invoker
	 *            - The user executing the command.
	 * @param object
	 *            - The object to be created.
	 * @throws MalformedStatementException
	 */
	public MySqlCreateViewStatement(IUser invoker, IView object) throws MalformedStatementException {
		super(invoker, object);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.implementation.abstraction.create.CreateStatement#toSQL()
	 */
	@Override
	public String toSQL() throws TranslationException {

		String columnList = "(";
		for (int i = 0; i < ((IView) this.getObject()).getCardinality(); i++) {
			if (i != 0)
				columnList += " , ";
			columnList += ((IView) this.getObject()).getFields().get(i);
		}
		columnList += ")";

		String cmd = "CREATE OR REPLACE ALGORITHM = MERGE VIEW " + this.getObject().getId() + " " + columnList + " AS " + ((IView) this.getObject()).getSelectQuery().toSQL();
		if (!cmd.endsWith(";"))
			cmd += ";";
		return cmd;

	}

	@Override
	public ICreateViewStatement copy() throws CopyException {
		try {
			return new MySqlCreateViewStatement(getInvoker(), getObject().copy());
		} catch (MalformedStatementException e) {
			throw new CopyException(e);
		}
	}
}
