package prototype.statements.implementation.postgresql.add;

import prototype.interfaces.CopyException;
import prototype.statements.MalformedStatementException;
import prototype.statements.abstractImplementation.add.AddUser;
import prototype.statements.interfaces.TranslationException;
import prototype.statements.interfaces.add.IAddUser;
import prototype.users.IUser;

/**
 * The concrete class representing CREATE USER statements for the PostgreSQl database.
 * 
 * 
 *
 */
public class PostgreSqlAddUser extends AddUser {

	/**
	 * Constructor.
	 * 
	 * @param invoker
	 *            - The invoking user.
	 * @param toAdd
	 *            - The user to add.
	 * @throws MalformedStatementException
	 */
	public PostgreSqlAddUser(IUser invoker, IUser toAdd) throws MalformedStatementException {
		super(invoker, toAdd);
	}

	/* (non-Javadoc)
	 * @see guarnieri.marco.ethz.statements.interfaces.IStatement#copy()
	 */
	@Override
	public IAddUser copy() throws CopyException {
		try {
			return new PostgreSqlAddUser(getInvoker().copy(), getUser().copy());
		} catch (MalformedStatementException e) {
			throw new CopyException(e);
		}
	}

	/* (non-Javadoc)
	 * @see guarnieri.marco.ethz.statements.abstractImplementation.add.AddUser#toSQL()
	 */
	@Override
	public String toSQL() throws TranslationException {
		return "CREATE USER "+getUser().getUsername();
	}

	

}
