package prototype.statements.implementation.postgresql.create;

import prototype.interfaces.CopyException;
import prototype.objects.ITable;
import prototype.objects.constraints.ForeignKeyConstraint;
import prototype.objects.constraints.IConstraint;
import prototype.statements.MalformedStatementException;
import prototype.statements.abstractImplementation.create.CreateTableStatement;
import prototype.statements.interfaces.TranslationException;
import prototype.statements.interfaces.create.ICreateTableStatement;
import prototype.users.IUser;

/**
 * The concrete class representing CREATE TABLE statements for the PostgreSql database.
 * 
 * 
 *
 */
public class PostgreSqlCreateTableStatement extends CreateTableStatement {

	/**
	 * Constructor.
	 * 
	 * @param invoker
	 *            - The user executing the command.
	 * @param object
	 *            - The object to be created.
	 * @throws MalformedStatementException
	 */
	public PostgreSqlCreateTableStatement(IUser invoker, ITable object) throws MalformedStatementException {
		super(invoker, object);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see guarnieri.marco.ethz.statements.implementation.abstraction.create.CreateStatement#toSQL()
	 */
	@Override
	public String toSQL() throws TranslationException {
		// String cmd = "CREATE TABLE " + this.getObject().getId() + " (";
		// for (int i = 0; i < ((ITable) this.getObject()).getCardinality(); i++) {
		// if (i != 0)
		// cmd += " , ";
		// cmd += " " + ((ITable) this.getObject()).getFields().get(i) + " VARCHAR(200) NOT NULL";
		// }
		//
		//
		// // we must create an index for any possible combination of attributes
		//
		//
		// for (int i = 0; i < ((ITable) this.getObject()).getCardinality(); i++) {
		// cmd += " , ";
		// cmd += " INDEX " + ((ITable) this.getObject()).getFields().get(i) + "_ind (" + ((ITable) this.getObject()).getFields().get(i) + ")";
		// }
		//
		// for (IConstraint c : ((ITable) this.getObject()).getConstraints())
		// cmd += ", " + c.toSql();
		// cmd += ") ENGINE InnoDB;";
		// System.out.println(cmd);
		// return cmd;

		String cmd = "CREATE TABLE " + this.getObject().getId() + " (";
		for (int i = 0; i < ((ITable) this.getObject()).getCardinality(); i++) {
			if (i != 0)
				cmd += " , ";
			cmd += " " + ((ITable) this.getObject()).getFields().get(i) + " text NOT NULL";
		}
		cmd += ");";

		for (IConstraint c : ((ITable) this.getObject()).getConstraints()) {
			cmd += "ALTER TABLE " + this.getObject().getId() + " ADD " + c.toSql() + ";";
		}
		return cmd;

	}

	@Override
	public ICreateTableStatement copy() throws CopyException {
		try {
			return new PostgreSqlCreateTableStatement(getInvoker(), getObject().copy());
		} catch (MalformedStatementException e) {
			throw new CopyException(e);
		}
	}
}
