package prototype.statements.implementation.postgresql.update;

import prototype.interfaces.CopyException;
import prototype.objects.ITable;
import prototype.queries.domain.relational.calculus.syntax.expressions.Constant;
import prototype.statements.MalformedStatementException;
import prototype.statements.abstractImplementation.update.DeleteStatement;
import prototype.statements.interfaces.TranslationException;
import prototype.statements.interfaces.update.IDeleteStatement;
import prototype.statements.modes.UPDATE_MODE;
import prototype.tuple.ITuple;
import prototype.users.IUser;

/**
 * The concrete class representing DELETE statements for the PostgreSql database.
 * 
 * 
 *
 */
public class PostgreSqlDeleteStatement extends DeleteStatement {

	/**
	 * Constructor.
	 * 
	 * @param invoker
	 *            - The user executing the statement.
	 * @param mode
	 *            - The update mode (DELETE)
	 * @param schema
	 *            - The relation schema the statement is going to modify.
	 * @param tuple
	 *            - The tuple the statement is going to add or remove.
	 * @throws MalformedStatementException
	 */
	public PostgreSqlDeleteStatement(IUser invoker, UPDATE_MODE mode, ITable schema, ITuple tuple) throws MalformedStatementException {
		super(invoker, mode, schema, tuple);
	}

	@Override
	public String toSQL() throws TranslationException {

		String condition = "";
		for (int i = 0; i < this.getRelationSchema().getCardinality(); i++) {
			if (i != 0)
				condition += " AND ";
			if (this.getTuple().getContent().get(i) instanceof Constant)
				condition += this.getRelationSchema().getFields().get(i) + " = \"" + ((Constant) this.getTuple().getContent().get(i)).getValue()
						+ "\"";
			else
				throw new TranslationException("The tuple contains free variable.");
		}

		return "DELETE FROM " + this.getRelationSchema().getId() + " WHERE " + condition + ";";

	}

	@Override
	public IDeleteStatement copy() throws CopyException {
		try {
			return new PostgreSqlDeleteStatement(getInvoker().copy(), getMode(), getRelationSchema().copy(), getTuple().copy());
		} catch (MalformedStatementException e) {
			throw new CopyException(e);
		}
	}

}
