package prototype.statements.implementation.postgresql.update;

import prototype.interfaces.CopyException;
import prototype.objects.ITable;
import prototype.queries.domain.relational.calculus.syntax.IOperand;
import prototype.queries.domain.relational.calculus.syntax.expressions.Constant;
import prototype.statements.MalformedStatementException;
import prototype.statements.abstractImplementation.update.InsertStatement;
import prototype.statements.interfaces.TranslationException;
import prototype.statements.interfaces.update.IInsertStatement;
import prototype.statements.modes.UPDATE_MODE;
import prototype.tuple.ITuple;
import prototype.users.IUser;
import prototype.util.Util;

/**
 * The concrete class representing INSERT statements for the PostgreSql database.
 * 
 * 
 *
 */
public class PostgreSqlInsertStatement extends InsertStatement {

	/**
	 * Constructor.
	 * 
	 * @param invoker
	 *            - The user executing the statement.
	 * @param mode
	 *            - The update mode (INSERT)
	 * @param schema
	 *            - The relation schema the statement is going to modify.
	 * @param tuple
	 *            - The tuple the statement is going to add or remove.
	 * @throws MalformedStatementException
	 */
	public PostgreSqlInsertStatement(IUser invoker, UPDATE_MODE mode, ITable schema, ITuple tuple) throws MalformedStatementException {
		super(invoker, mode, schema, tuple);
	}

	@Override
	public String toSQL() throws TranslationException {

		String columns = "( ";
		for (String c : this.getRelationSchema().getFields())
			columns += " " + c + " ,";
		columns = Util.replaceLast(columns, ",", " )");

		String values = "( ";
		for (IOperand v : this.getTuple().getContent()) {
			if (v instanceof Constant)
				values += " " +  v + " ,";
			else
				throw new TranslationException("The tuple contains free variable.");
		}

		values = Util.replaceLast(values, " ,", " )");

		return "INSERT INTO " + this.getRelationSchema().getId() + " " + columns + " VALUES " + values + ";";

	}

	@Override
	public IInsertStatement copy() throws CopyException {
		try {
			return new PostgreSqlInsertStatement(getInvoker().copy(), getMode(), getRelationSchema().copy(), getTuple().copy());
		} catch (MalformedStatementException e) {
			throw new CopyException(e);
		}
	}

}
