package prototype.statements.interfaces;

import prototype.interfaces.CopyException;
import prototype.interfaces.ICopiable;
import prototype.interfaces.IExecutable;
import prototype.users.IUser;

/**
 * Interface for all statements supported by our prototype.
 * 
 * 
 * 
 *
 */
public interface IStatement extends ICopiable, IExecutable {

	/**
	 * Translate the statement to SQL code.
	 * 
	 * @return The SQL statement.
	 */
	public String toSQL() throws TranslationException;

	/**
	 * Get the user invoking the command.
	 * 
	 * @return the user invoking the command.
	 */
	public IUser getInvoker();

	/**
	 * Get the textual representation of the statement's definition.
	 * 
	 * @return The statement's textual representation.
	 */
	public String getDefinition();

	
	public IStatement copy() throws CopyException;
}
