package prototype.statements.interfaces.changePolicy;

import prototype.interfaces.CopyException;
import prototype.privileges.IPrivilege;
import prototype.statements.interfaces.IStatement;
import prototype.statements.modes.CHANGE_POLICY_MODE;
import prototype.users.IUser;

/**
 * Interface for GRANT, GRANT WITH GRANT OPTION, and REVOKE statements.
 * 
 * 
 *
 */
public interface IChangePolicyStatement extends IStatement {

	/**
	 * Get the privilege used in the statement.
	 * 
	 * @return the privilege used in the statement.
	 */
	public IPrivilege getPrivilege();

	/**
	 * Get the user that is the target of the statement.
	 * 
	 * @return the user that is the target of the statement.
	 */
	public IUser getTarget();

	/**
	 * Get the mode associated with the statement.
	 * 
	 * @return GRANT if the stmt is a GRANT statement, GRANT_WITH_OPTION if the stmt is a GRANT WITH GRANT OPTION statement, and REVOKE if the stmt is
	 *         a REVOKE statement.
	 */
	public CHANGE_POLICY_MODE getMode();
	
	
	/* (non-Javadoc)
	 * @see guarnieri.marco.ethz.statements.interfaces.IStatement#copy()
	 */
	public IChangePolicyStatement copy() throws CopyException;

}
