package prototype.system.interfaces;

import java.util.Set;
import java.util.SortedSet;

import prototype.objects.ITable;
import prototype.objects.ITrigger;
import prototype.objects.IView;
import prototype.statements.interfaces.changePolicy.IPositiveGrantStatement;
import prototype.system.MalformedStateException;
import prototype.tuple.ITuple;
import prototype.users.IUser;

/**
 * Interface for the state factory.
 * 
 * 
 *
 */
public interface IStateFactory {

	/**
	 * Create a new state.
	 * 
	 * @param users
	 *            - The users.
	 * @param tables
	 *            - The tables.
	 * @param policy
	 *            - The policy.
	 * @param views
	 *            - The set of views.
	 * @param triggers
	 *            - The set of triggers.
	 * @param invoker
	 *            - The invoker.
	 * @param rollback
	 *            - The rollback state
	 * @param scheduledTriggers
	 *            - The scheduled triggers
	 * @param tuple
	 *            - The current tuple.
	 * @param trigger
	 *            - The trigger that the system is executing.
	 * @throws MalformedStateException
	 */
	public IState createNewState(Set<IUser> users, Set<ITable> tables, Set<IPositiveGrantStatement> policy, Set<IView> views, Set<ITrigger> triggers,
			IUser invoker, IState rollback, SortedSet<ITrigger> scheduledTriggers, ITuple tuple, ITrigger trigger) throws MalformedStateException;

}
