package prototype.system.mysql;

import java.sql.Savepoint;
import java.util.Set;
import java.util.SortedSet;

import prototype.objects.ITable;
import prototype.objects.ITrigger;
import prototype.objects.IView;
import prototype.statements.interfaces.changePolicy.IPositiveGrantStatement;
import prototype.system.MalformedStateException;
import prototype.system.implementation.AbstractState;
import prototype.system.interfaces.IState;
import prototype.tuple.ITuple;
import prototype.users.IUser;

/**
 * Class representing MySql states.
 * 
 * 
 *
 */
public class MySqlState extends AbstractState {

	/**
	 * The savepoint associated with the state.
	 */
	private Savepoint savePoint;

	/**
	 * Constructor.
	 * 
	 * @param users
	 *            - The users.
	 * @param tables
	 *            - The tables.
	 * @param policy
	 *            - The policy.
	 * @param views
	 *            - The set of views.
	 * @param triggers
	 *            - The set of triggers.
	 * @param savepoint
	 *            - The savepoint.
	 * @throws MalformedStateException
	 */
	public MySqlState(Set<IUser> users, Set<ITable> tables, Set<IPositiveGrantStatement> policy, Set<IView> views, Set<ITrigger> triggers,
			Savepoint savepoint) throws MalformedStateException {
		super(users, tables, policy, views, triggers);
		if (savepoint == null)
			throw new MalformedStateException("Savepoint cannot be null.");
		else
			this.savePoint = savepoint;
	}

	/**
	 * Constructor.
	 * 
	 * @param users
	 *            - The users.
	 * @param tables
	 *            - The tables.
	 * @param policy
	 *            - The policy.
	 * @param views
	 *            - The set of views.
	 * @param triggers
	 *            - The set of triggers.
	 * @param invoker
	 *            - The invoker.
	 * @param rollback
	 *            - The rollback state
	 * @param scheduledTriggers
	 *            - The scheduled triggers.
	 * @param tuple
	 *            - The current tuple.
	 * @param savepoint
	 *            - The savepoint.
	 * @throws MalformedStateException
	 */
	public MySqlState(Set<IUser> users, Set<ITable> tables, Set<IPositiveGrantStatement> policy, Set<IView> views, Set<ITrigger> triggers,
			IUser invoker, IState rollback, SortedSet<ITrigger> scheduledTriggers, ITuple tuple, ITrigger trigger, Savepoint savepoint) throws MalformedStateException {
		super(users, tables, policy, views, triggers, invoker, rollback, scheduledTriggers, tuple, trigger);
		if (savepoint == null)
			throw new MalformedStateException("Savepoint cannot be null.");
		else
			this.savePoint = savepoint;
	}

	/**
	 * Get the savepoint associated with this state.
	 * 
	 * @return The savepoint.
	 */
	public Savepoint getSavePoint() {
		return savePoint;
	}

}
