package prototype.system.mysql;

import java.sql.Savepoint;
import java.util.Set;
import java.util.SortedSet;

import prototype.database.mysql.MySqlConnectionManager;
import prototype.objects.ITable;
import prototype.objects.ITrigger;
import prototype.objects.IView;
import prototype.statements.interfaces.changePolicy.IPositiveGrantStatement;
import prototype.system.MalformedStateException;
import prototype.system.implementation.ExecutionException;
import prototype.system.interfaces.IState;
import prototype.system.interfaces.IStateFactory;
import prototype.tuple.ITuple;
import prototype.users.IUser;

/**
 * State factory for MySql.
 * 
 * 
 *
 */
public class MySqlStateFactory implements IStateFactory {

	/**
	 * The connection manager.
	 */
	private MySqlConnectionManager manager;

	/**
	 * Constructor.
	 * 
	 * @throws ExecutionException
	 */
	public MySqlStateFactory() {

	}

	/**
	 * Set the connection manager.
	 * 
	 * @param manager
	 *            - The manager.
	 */
	public void setConnectionManager(MySqlConnectionManager manager) {
		this.manager = manager;
	}

	@Override
	public MySqlState createNewState(Set<IUser> users, Set<ITable> tables, Set<IPositiveGrantStatement> policy, Set<IView> views,
			Set<ITrigger> triggers, IUser invoker, IState rollback, SortedSet<ITrigger> scheduledTriggers, ITuple tuple, ITrigger trigger)
			throws MalformedStateException {
		if (rollback == null || rollback instanceof MySqlState)

			try {
				// create the new savepoint
				String id = manager.getNewSavepointId();

				Savepoint save = manager.createNewSavepoint(id);

				return new MySqlState(users, tables, policy, views, triggers, invoker, rollback, scheduledTriggers, tuple, trigger, save);
			} catch (ExecutionException e) {
				throw new MalformedStateException(e);
			}
		else
			throw new MalformedStateException("Rollback must be a MySqlState");
	}

}
