package prototype.tuple;

import java.util.List;
import java.util.Vector;

import prototype.interfaces.CopyException;
import prototype.queries.MalformedExpressionException;
import prototype.queries.domain.relational.calculus.syntax.IOperand;
import prototype.util.Util;

/**
 * Class representing tuples.
 * 
 * 
 *
 */
public class Tuple implements ITuple {

	/**
	 * The tuple's content
	 */
	private List<IOperand> values;

	/**
	 * Constructor for the empty tuple.
	 */
	public Tuple() {
		this.values = new Vector<IOperand>();
	}

	/**
	 * Constructor.
	 * 
	 * @param values
	 *            - the values in the tuple.
	 */
	public Tuple(List<IOperand> values) {
		this();
		for (IOperand v : values)
			try {
				this.values.add(v.copy());
			} catch (MalformedExpressionException e) {
				assert (false); // unreachable
			}
	}

	/**
	 * Get the tuple's size.
	 * 
	 * @return The integer representing the tuple's size.
	 */
	public int getSize() {
		return this.values.size();
	}

	/**
	 * Get the tuple's content.
	 * 
	 * @return The tuple's content.
	 */
	public List<IOperand> getContent() {
		return this.values;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String res = "[";
		for (IOperand o : this.values)
			res += o + ", ";
		res = Util.replaceLast(res, ", ", "]");
		return res;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		else if (obj instanceof Tuple) {
			if (obj == this)
				return true;
			else
				return ((Tuple) obj).getContent().equals(this.getContent());
		} else
			return false;
	}
	
	@Override
	public int hashCode() {
		int result = 17;
		result = 31 * result + getContent().hashCode();
		return result;
	}

	@Override
	public ITuple copy() throws CopyException {
		List<IOperand> newValues = new Vector<IOperand>();
		for (IOperand op : values)
			try {
				newValues.add(op.copy());
			} catch (MalformedExpressionException e) {
				throw new CopyException(e);
			}
		return new Tuple(values);
	}
}
