package prototype.util;

/**
 * 
 *
 * @param <T>
 *            - First type in the pair.
 * @param <U>
 *            - Second type in the pair.
 */
public class Pair<T, U> {
	private final T first;
	private final U second;
	private transient final int hash;

	public Pair(T f, U s) {
		this.first = f;
		this.second = s;
		hash = (first == null ? 0 : first.hashCode() * 31) + (second == null ? 0 : second.hashCode());
	}

	public T getFirst() {
		return first;
	}

	public U getSecond() {
		return second;
	}

	@Override
	public int hashCode() {
		return hash;
	}
	
	@Override
	public boolean equals(Object oth) {
		if (this == oth) {
			return true;
		}
		if (oth == null || !(getClass().isInstance(oth))) {
			return false;
		}
		if(oth instanceof Pair<?,?>)
		{
			Pair<?, ?> other = (Pair<?, ?>) oth;
			return (first == null ? other.first == null : first.equals(other.first))
					&& (second == null ? other.second == null : second.equals(other.second));
		}
		else
			return false;
		

	}
	
	@Override
	public String toString() {
		
		return "FIRST: "+first+" SECOND: "+second;
	}

}
