package prototype.util;

import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import java.util.Vector;

public class Util {

	/**
	 * Replace the last occurrence of a string in another string
	 * 
	 * @param string
	 * @param toReplace
	 * @param replacement
	 * @return
	 */
	public static String replaceLast(String string, String toReplace, String replacement) {
		int pos = string.lastIndexOf(toReplace);
		if (pos > -1) {
			return string.substring(0, pos) + replacement + string.substring(pos + toReplace.length(), string.length());
		} else {
			return string;
		}
	}
	
	
	/**
	 * Compute the powerset of the given set. It returns the powerset in a list to preserve the "ordering" on the size.
	 * 
	 * @param tables
	 * @return
	 */
	public static <X> List<Set<X>> powerset(Set<X> variables) {
		List<Set<X>> ps = new Vector<Set<X>>();
		ps.add(new HashSet<X>()); // add the empty set

		// for every item in the original list
		for (X var : variables) {
			List<Set<X>> newPs = new Vector<Set<X>>();

			for (Set<X> subset : ps) {
				// copy all of the current powerset's subsets
				newPs.add(subset);

				// plus the subsets appended with the current item
				LinkedHashSet<X> newSubset = new LinkedHashSet<X>(subset);
				newSubset.add(var);
				newPs.add(newSubset);
			}

			// powerset is now powerset of list.subList(0, list.indexOf(item)+1)
			ps = newPs;
		}
		return ps;

	}
	
}
